/**
 * @overview WMF Command Class
 * @author Jan "Waffle" Christophersen
 * @copyright (c) Jan "Waffle" Christophersen
 */

'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

var colors = {
  red: new RGB(255, 0, 0),
  orange: new RGB(243, 156, 18)
};

/**
 * This function is called before executing a command. It can stop the command from being executed beforehand.
 *
 * @callback WMFCommand~validator
 * @param {Player} player - the player that executes the command
 * @returns {string|boolean} return a string to display it as an error message. return false to send the USAGE message. return true to pass the command to the handle function.
 */

/**
 * This function handles a command.
 *
 * @callback WMFCommand~handler
 * @param {Player} player - the player that executes the command
 * @param {*} ...args - command arguments
 */

/**
 * the WMFCommand class represents a command that can be used via GTA:MP's chat. It has one or multiple triggers.
 */

var WMFCommand = (function () {
  /**
   * Creates a new WMFCommand.
   *
   * @param {Array.<string>} trigger - command triggers
   * @param {WMFCommand~handler} handler - command handler
   * @param {Object} [options] - options
   * @param {Array.<Array.<string>>} [options.parameter] - command parameters
   * @param {WMFCommand~validator} [options.validate] - validator before executing a command
   */

  function WMFCommand(trigger, handler) {
    var options = arguments.length <= 2 || arguments[2] === undefined ? { parameter: [], validate: function validate() {
        return true;
      } } : arguments[2];

    _classCallCheck(this, WMFCommand);

    if (trigger.length === 0) {
      throw new Error('no trigger given in command.');
    }
    var parameter = options.parameter;
    var validate = options.validate;

    /** @type {Set} */
    this.trigger = new Set(trigger);
    /** @type {WMFCommand~handler} */
    this.handler = handler;
    /** @type {Object} */
    this.options = options;
    /** @type {Array.<Array.<string>>} */
    this.parameter = parameter;
    /** @type {WMFCommand~validator} */
    this.validator = validate;

    /** @type {string} */
    this.usage = '';

    /** @type {number} */
    this.minParameterCount = 0;
    /** @type {number} */
    this.optionalParameterCount = 0;

    // parameter validation
    for (var idx = 0; idx < parameter.length; idx++) {
      if (parameter[idx].length < 2) {
        throw new Error('in parameter \'' + parameter[idx] + '\': type missing');
      }

      if (parameter[idx][1] === 'text' && idx < parameter.length - 1) {
        throw new Error('in parameter \'' + parameter[idx] + '\': \'text\' must always be the last parameter');
      }

      if (parameter[idx].length === 3) {
        this.optionalParameterCount++;
      } else {
        if (this.optionalParameterCount > 0) {
          // i c what u did there
          throw new Error('in parameter \'' + parameter[idx] + '\': non-optional parameter after optional parameter');
        }
        this.minParameterCount++;
      }
    }

    this._generateUsageMessage();
  }

  /**
   * Generates the usage message for the command
   */

  _createClass(WMFCommand, [{
    key: '_generateUsageMessage',
    value: function _generateUsageMessage() {
      var _this = this;

      this.usage = 'USAGE: /';

      if (this.trigger.size > 1) {
        this.usage += '[';
      }
      this.trigger.forEach(function (trigger) {
        _this.usage += trigger + '|';
      });
      this.usage = this.usage.substr(0, this.usage.length - 1);

      if (this.trigger.size > 1) {
        this.usage += ']';
      }
      this.usage += ' ';

      this.parameter.forEach(function (parameter, idx) {
        if (idx === _this.minParameterCount) {
          _this.usage += '( ';
        }
        if (idx >= _this.minParameterCount) {
          _this.usage += '[' + parameter[0] + ' = ' + parameter[2] + '] ';
        } else {
          _this.usage += '[' + parameter[0] + '] ';
        }
      });
      if (this.optionalParameterCount > 0) {
        this.usage += ')';
      }
    }

    /**
     * Executes the command.
     *
     * @param {Player} player - executing player
     * @param {Array.<string>} args - arguments
     */
  }, {
    key: 'execute',
    value: function execute(player, args) {
      // Before doing anything, we should check for a validator.
      if (this.validator) {
        var validation = this.validator(player);

        if (validation === false) {
          player.SendChatMessage('You are not allowed to use this command.', colors.red);
          return;
        } else if (typeof validation === 'string') {
          player.SendChatMessage(validation, colors.red);
          return;
        }
      }

      // check parameter count
      if (args.length < this.minParameterCount) {
        player.SendChatMessage(this.usage, colors.orange);
        return;
      }

      // parse arguments
      var parse = function parse(arg, param) {
        // TODO 'player' and 'target' type
        switch (param[1]) {
          case 'string':
            return { value: arg };
          case 'integer':
          case 'int':
            var int = parseInt(arg, 10);
            if (isNaN(int)) {
              return { err: NaN };
            }
            return { value: int };
          case 'float':
            var float = parseFloat(arg, 10);
            if (isNaN(float)) {
              return { err: NaN };
            }
            return { value: float };
          default:
            log.warn('cannot parse unknown type \'' + param[1] + '\'');
            return { err: 'invalid type' };
        }
      };

      var parsedArgs = [];
      for (var idx = 0; idx < args.length; idx++) {
        if (typeof this.parameter[idx] === 'undefined') {
          break;
        }

        if (this.parameter[idx][1] === 'text') {
          parsedArgs.push(args.splice(idx, args.length).join(' '));
          break;
        }

        var parsed = parse(args[idx], this.parameter[idx]);
        if (parsed.err) {
          player.SendChatMessage(this.usage, colors.orange);
          return;
        }

        parsedArgs.push(parsed.value);
      }

      if (parsedArgs.length < this.minParameterCount + this.optionalParameterCount) {
        for (var idx = parsedArgs.length; idx < this.minParameterCount + this.optionalParameterCount - parsedArgs.length; idx++) {
          if (typeof this.parameter[idx] === 'undefined') {
            break;
          }

          parsedArgs.push(this.parameter[idx][2]);
        }
      }

      this.handler.apply(this, [player].concat(parsedArgs));
    }
  }]);

  return WMFCommand;
})();

exports['default'] = WMFCommand;
module.exports = exports['default'];
//# sourceMappingURL=wmfCommand.js.map