/**
 * @overview WMF Permission Container
 * @author Jan "Waffle" Christophersen
 * @copyright (c) Jan "Waffle" Christophersen
 */

/**
 * Looks up a WMFPermission.
 *
 * @callback {WMFPermissionContainer~lookup}
 * @param {string} name - permission name
 * @returns {WMFPermission|null}
 */

/**
 * A WMFPermissionContainer contains.... permissions. It allows shorthand-methods to add, remove or evaluate a permission.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var WMFPermissionContainer = (function () {
  /**
   * Creates a new PermissionContainer.
   *
   * @param {WMFPermissionContainer~lookup} - function to look up a permission
   */

  function WMFPermissionContainer(lookup) {
    _classCallCheck(this, WMFPermissionContainer);

    /** @type {Set.<string>} */
    this.permissions = new Set();
    /** @type {WMFPermissionContainer~lookup} */
    this._lookup = lookup;
  }

  /**
   * Adds a Permission to the container.
   *
   * @param {string} name - permission name
   * @returns {boolean} - whether the permission could be added
   */

  _createClass(WMFPermissionContainer, [{
    key: "add",
    value: function add(name) {
      if (this.has(name)) {
        return true;
      }
      var perm = this._lookup(name);
      if (!perm) {
        return false;
      }
      this.permissions.add(name);
    }

    /**
     * Evaluates a permission. The result of the evaluator decides whether the permission will be added or removed from the Container.
     *
     * @see {WMFPermission#evaluate}
     * @param {string} name - permission name
     * @param {*} ...args - arguments
     * @returns {boolean} evaluator result
     */
  }, {
    key: "evaluate",
    value: function evaluate(name) {
      var perm = this._lookup(name);
      if (!perm) {
        log.warn("trying to evaluate unknown permission '" + name + "'");
        return false;
      }

      for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
        args[_key - 1] = arguments[_key];
      }

      var result = perm.evaluate.apply(perm, args);

      if (result) {
        if (!this.has(name)) {
          if (!this.add(name)) {
            throw new Error("internal error while trying to add permission '" + name + "' after evaluating");
          }
        }
      } else {
        if (this.has(name)) {
          if (!this.remove(name)) {
            throw new Error("internal error while trying to remove permission '" + name + "' after evaluating");
          }
        }
      }
    }

    /**
     * Returns whether the Container has the given permission.
     *
     * @param {string} name - name
     * @returns {boolean}
     */
  }, {
    key: "has",
    value: function has(name) {
      return this.permissions.has(name);
    }

    /**
     * Removes a Permission.
     *
     * @param {string} name - permission name
     */
  }, {
    key: "remove",
    value: function remove(name) {
      return this.permissions["delete"](name);
    }
  }]);

  return WMFPermissionContainer;
})();

exports["default"] = WMFPermissionContainer;
module.exports = exports["default"];
//# sourceMappingURL=wmfPermissionContainer.js.map